#include <msp430.h>
#include "SS1BTPS.h"             /* Main SS1 Bluetooth Stack Header.          */
#include "BTPSKRNL.h"            /* BTPS Kernel Header.                       */
#include "SPP.h"             /* Application Header.                       */
#include "ADC.h"

#define MAX_INQUIRY_RESULTS 5

#define MAX_SUPPORTED_LINK_KEYS 1

#define DEFAULT_PIN_CODE "0000"

#define FUNCTION_ERROR -4

#define INVALID_PARAMETERS_ERROR -6

#define UNABLE_TO_INITIALIZE_STACK -7

#define INVALID_STACK_ID_ERROR -8

#define UNABLE_TO_REGISTER_SERVER -9

/* Following converts a Sniff Parameter in Milliseconds to frames.   */
#define MILLISECONDS_TO_BASEBAND_SLOTS(_x)   ((_x) / (0.625))

/* The following type definition represents the container type which */
/* holds the mapping between Bluetooth devices (based on the BD_ADDR)*/
/* and the Link Key (BD_ADDR <-> Link Key Mapping).                  */
typedef struct _tagLinkKeyInfo_t {
	BD_ADDR_t BD_ADDR;
	Link_Key_t LinkKey;
} LinkKeyInfo_t;

/* NOTE: these inquiry results may not be needed as sniffing is not a
 * feature of our app anymore
 */

/* valid inquiry responses */
static unsigned int NumberofValidResponses;

/* inqiry responses */
static BD_ADDR_t InquiryResultList[MAX_INQUIRY_RESULTS];

/* NOTE: these are usually 1, but they are used as semaphores for
 * whether or not we are connected
 */

/* handle for the bluetooth stack */
static unsigned int BluetoothStackID;
/* handle for the serial port */
static int SerialPortID;
/* handle for the server port */
static int ServerPortID;

/* handle of the most recent connection */
static Word_t Connection_Handle;

/* handle for the service record for the spp with the sdp */
static DWord_t SPPServerSDPHandle;

/* holds BD_ADDR <-> Link Keys for pairing */
static LinkKeyInfo_t LinkKeyInfo[MAX_SUPPORTED_LINK_KEYS];

/* the current remote bluetooth address */
static BD_ADDR_t CurrentRemoteBD_ADDR;
/* address of the selected device */
static BD_ADDR_t SelectedBD_ADDR;
/* NULL address for comparison */
static BD_ADDR_t NullADDR;

/* buffer of data if we try and write and we can't write */
static unsigned int BufferLength;
static unsigned char Buffer[256];

static int OpenStack(HCI_DriverInformation_t *HCI_DriverInformation,
		BTPS_Initialization_t *BTPS_Initialization);
static int CloseStack(void);

static int SetDisc(void);
static int SetConnect(void);
static int SetPairable(void);
static int DeleteLinkKey(BD_ADDR_t BD_ADDR);

/* BTPS Callback function prototypes.                                */
static void BTPSAPI GAP_Event_Callback(unsigned int BluetoothStackID,
		GAP_Event_Data_t *GAP_Event_Data, unsigned long CallbackParameter);
static void BTPSAPI SPP_Event_Callback(unsigned int BluetoothStackID,
		SPP_Event_Data_t *SPP_Event_Data, unsigned long CallbackParameter);
static void BTPSAPI HCI_Event_Callback(unsigned int BluetoothStackID,
		HCI_Event_Data_t *HCI_Event_Data, unsigned long CallbackParameter);

/* The following function is responsible for opening the SS1         */
/* Bluetooth Protocol Stack.  This function accepts a pre-populated  */
/* HCI Driver Information structure that contains the HCI Driver     */
/* Transport Information.  This function returns zero on successful  */
/* execution and a negative value on all errors.                     */
static int OpenStack(HCI_DriverInformation_t *HCI_DriverInformation,
		BTPS_Initialization_t *BTPS_Initialization) {
	int Result;
	int ret_val = 0;
	Byte_t Status;
	BD_ADDR_t BD_ADDR;
	L2CA_Link_Connect_Params_t L2CA_Link_Connect_Params;

	/* First check to see if the Stack has already been opened.          */
	if (!BluetoothStackID) {
		/* Next, makes sure that the Driver Information passed appears to */
		/* be semi-valid.                                                 */
		if ((HCI_DriverInformation) && (BTPS_Initialization)) {
			Display(("\r\n"));

			/* Initialize BTPSKNRl.                                        */
			BTPS_Init(BTPS_Initialization);

			Display(("OpenStack().\r\n"));

			/* Initialize the Stack                                        */
			Result = BSC_Initialize(HCI_DriverInformation, 0);

			/* Next, check the return value of the initialization to see if*/
			/* it was successful.                                          */
			if (Result > 0) {
				/* The Stack was initialized successfully, inform the user  */
				/* and set the return value of the initialization function  */
				/* to the Bluetooth Stack ID.                               */
				BluetoothStackID = Result;
				Display(("Bluetooth Stack ID: %d\r\n", BluetoothStackID));

				/* Let's output the Bluetooth Device Address so that the    */
				/* user knows what the Device Address is.                   */
				GAP_Query_Local_BD_ADDR(BluetoothStackID, &BD_ADDR);

				/* Go ahead and allow Master/Slave Role Switch.             */
				L2CA_Link_Connect_Params.L2CA_Link_Connect_Request_Config =
						cqAllowRoleSwitch;
				L2CA_Link_Connect_Params.L2CA_Link_Connect_Response_Config =
						csMaintainCurrentRole;

				L2CA_Set_Link_Connection_Configuration(BluetoothStackID,
						&L2CA_Link_Connect_Params);

				if (HCI_Command_Supported(BluetoothStackID,
				HCI_SUPPORTED_COMMAND_WRITE_DEFAULT_LINK_POLICY_BIT_NUMBER) > 0)
					HCI_Write_Default_Link_Policy_Settings(BluetoothStackID,
							(HCI_LINK_POLICY_SETTINGS_ENABLE_MASTER_SLAVE_SWITCH
									| HCI_LINK_POLICY_SETTINGS_ENABLE_SNIFF_MODE),
							&Status);

				/* Delete all Stored Link Keys.                             */
				ASSIGN_BD_ADDR(BD_ADDR, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);

				DeleteLinkKey(BD_ADDR);
			} else {
				/* The Stack was NOT initialized successfully, inform the   */
				/* user and set the return value of the initialization      */
				/* function to an error.                                    */

				BluetoothStackID = 0;

				ret_val = UNABLE_TO_INITIALIZE_STACK;
			}
		} else {
			/* One or more of the necessary parameters are invalid.        */
			ret_val = INVALID_PARAMETERS_ERROR;
		}
	}

	return (ret_val);
}

/* The following function is responsible for closing the SS1         */
/* Bluetooth Protocol Stack.  This function requires that the        */
/* Bluetooth Protocol stack previously have been initialized via the */
/* OpenStack() function.  This function returns zero on successful   */
/* execution and a negative value on all errors.                     */
static int CloseStack(void) {
	int ret_val = 0;

	/* First check to see if the Stack has been opened.                  */
	if (BluetoothStackID) {
		/* Simply close the Stack                                         */
		BSC_Shutdown(BluetoothStackID);

		/* Free BTPSKRNL allocated memory.                                */
		BTPS_DeInit();

		Display(("Stack Shutdown.\r\n"));

		/* Flag that the Stack is no longer initialized.                  */
		BluetoothStackID = 0;

		/* Flag success to the caller.                                    */
		ret_val = 0;
	} else {
		/* A valid Stack ID does not exist, inform to user.               */
		ret_val = UNABLE_TO_INITIALIZE_STACK;
	}

	return (ret_val);
}

/* The following function is responsible for placing the Local       */
/* Bluetooth Device into General Discoverablity Mode.  Once in this  */
/* mode the Device will respond to Inquiry Scans from other Bluetooth*/
/* Devices.  This function requires that a valid Bluetooth Stack ID  */
/* exists before running.  This function returns zero on successful  */
/* execution and a negative value if an error occurred.              */
static int SetDisc(void) {
	int ret_val = 0;

	/* First, check that a valid Bluetooth Stack ID exists.              */
	if (BluetoothStackID) {
		/* A semi-valid Bluetooth Stack ID exists, now attempt to set the */
		/* attached Devices Discoverablity Mode to General.               */
		ret_val = GAP_Set_Discoverability_Mode(BluetoothStackID,
				dmGeneralDiscoverableMode, 0);

		/* Next, check the return value of the GAP Set Discoverability    */
		/* Mode command for successful execution.                         */
		if (ret_val) {
			/* An error occurred while trying to set the Discoverability   */
			/* Mode of the Device.                                         */
		}
	} else {
		/* No valid Bluetooth Stack ID exists.                            */
		ret_val = INVALID_STACK_ID_ERROR;
	}

	return (ret_val);
}

/* The following function is responsible for placing the Local       */
/* Bluetooth Device into Connectable Mode.  Once in this mode the    */
/* Device will respond to Page Scans from other Bluetooth Devices.   */
/* This function requires that a valid Bluetooth Stack ID exists     */
/* before running.  This function returns zero on success and a      */
/* negative value if an error occurred.                              */
static int SetConnect(void) {
	int ret_val = 0;

	/* First, check that a valid Bluetooth Stack ID exists.              */
	if (BluetoothStackID) {
		/* Attempt to set the attached Device to be Connectable.          */
		ret_val = GAP_Set_Connectability_Mode(BluetoothStackID,
				cmConnectableMode);

		/* Next, check the return value of the                            */
		/* GAP_Set_Connectability_Mode() function for successful          */
		/* execution.                                                     */
		if (ret_val) {
			/* An error occurred while trying to make the Device           */
			/* Connectable.                                                */
		}
	} else {
		/* No valid Bluetooth Stack ID exists.                            */
		ret_val = INVALID_STACK_ID_ERROR;
	}

	return (ret_val);
}

/* The following function is responsible for placing the local       */
/* Bluetooth device into Pairable mode.  Once in this mode the device*/
/* will response to pairing requests from other Bluetooth devices.   */
/* This function returns zero on successful execution and a negative */
/* value on all errors.                                              */
static int SetPairable(void) {
	int Result;
	int ret_val = 0;

	/* First, check that a valid Bluetooth Stack ID exists.              */
	if (BluetoothStackID) {
		/* Attempt to set the attached device to be pairable.             */
		Result = GAP_Set_Pairability_Mode(BluetoothStackID, pmPairableMode);

		/* Next, check the return value of the GAP Set Pairability mode   */
		/* command for successful execution.                              */
		if (!Result) {
			/* The device has been set to pairable mode, now register an   */
			/* Authentication Callback to handle the Authentication events */
			/* if required.                                                */
			Result = GAP_Register_Remote_Authentication(BluetoothStackID,
					GAP_Event_Callback, (unsigned long) 0);

			/* Next, check the return value of the GAP Register Remote     */
			/* Authentication command for successful execution.            */
			if (Result) {
				/* An error occurred while trying to execute this function. */

				ret_val = Result;
			}
		} else {
			/* An error occurred while trying to make the device pairable. */

			ret_val = Result;
		}
	} else {
		/* No valid Bluetooth Stack ID exists.                            */
		ret_val = INVALID_STACK_ID_ERROR;
	}

	return (ret_val);
}

/* The following function is a utility function that exists to delete*/
/* the specified Link Key from the Local Bluetooth Device.  If a NULL*/
/* Bluetooth Device Address is specified, then all Link Keys will be */
/* deleted.                                                          */
static int DeleteLinkKey(BD_ADDR_t BD_ADDR) {
	int Result;
	Byte_t Status_Result;
	Word_t Num_Keys_Deleted = 0;
	BD_ADDR_t NULL_BD_ADDR;

	Result = HCI_Delete_Stored_Link_Key(BluetoothStackID, BD_ADDR, TRUE,
			&Status_Result, &Num_Keys_Deleted);

	/* Any stored link keys for the specified address (or all) have been */
	/* deleted from the chip.  Now, let's make sure that our stored Link */
	/* Key Array is in sync with these changes.                          */

	/* First check to see all Link Keys were deleted.                    */
	ASSIGN_BD_ADDR(NULL_BD_ADDR, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);

	if (COMPARE_BD_ADDR(BD_ADDR, NULL_BD_ADDR))
		BTPS_MemInitialize(LinkKeyInfo, 0, sizeof(LinkKeyInfo));
	else {
		/* Individual Link Key.  Go ahead and see if know about the entry */
		/* in the list.                                                   */
		for (Result = 0; (Result < sizeof(LinkKeyInfo) / sizeof(LinkKeyInfo_t));
				Result++) {
			if (COMPARE_BD_ADDR(BD_ADDR, LinkKeyInfo[Result].BD_ADDR)) {
				LinkKeyInfo[Result].BD_ADDR = NULL_BD_ADDR;

				break;
			}
		}
	}

	return (Result);
}

/* The following function is responsible for opening a Serial Port   */
/* Server on the Local Device.  This function opens the Serial Port  */
/* Server on the first RFCOMM Channel.                               */
static int OpenServerOnOne(void) {
	int ret_val;
	char *ServiceName;

	/* First check to see if a valid Bluetooth Stack ID exists.          */
	if (BluetoothStackID) {
		/* Make sure that there is not already a Serial Port Server open. */
		if (!ServerPortID) {

			/* Simply attempt to open an Serial Server, on RFCOMM Server*/
			/* Port 1.                                                  */
			ret_val = SPP_Open_Server_Port(BluetoothStackID, 1,
					SPP_Event_Callback, (unsigned long) 0);

			/* If the Open was successful, then note the Serial Port    */
			/* Server ID.                                               */
			if (ret_val > 0) {
				/* Note the Serial Port Server ID of the opened Serial   */
				/* Port Server.                                          */
				ServerPortID = ret_val;

				/* Create a Buffer to hold the Service Name.             */
				if ((ServiceName = BTPS_AllocateMemory(64)) != NULL) {
					/* The Server was opened successfully, now register a */
					/* SDP Record indicating that an Serial Port Server   */
					/* exists. Do this by first creating a Service Name.  */
					BTPS_SprintF(ServiceName, "Serial Port Server Port 1");

					/* Now that a Service Name has been created try to    */
					/* Register the SDP Record.                           */
					ret_val = SPP_Register_Generic_SDP_Record(BluetoothStackID,
							ServerPortID, ServiceName, &SPPServerSDPHandle);

					/* If there was an error creating the Serial Port     */
					/* Server's SDP Service Record then go ahead an close */
					/* down the server an flag an error.                  */
					if (ret_val < 0) {
						Display(
								("Unable to Register Server SDP Record, Error = %d.\r\n", ret_val));

						SPP_Close_Server_Port(BluetoothStackID, ServerPortID);

						/* Flag that there is no longer an Serial Port     */
						/* Server Open.                                    */
						ServerPortID = 0;

						ret_val = UNABLE_TO_REGISTER_SERVER;
					} else {
						/* Simply flag to the user that everything         */
						/* initialized correctly.                          */
						Display(("Server Opened: 1.\r\n"));

						/* Flag success to the caller.                     */
						ret_val = 0;
					}

					/* Free the Service Name buffer.                      */
					BTPS_FreeMemory(ServiceName);
				} else {
					Display(
							("Failed to allocate buffer to hold Service Name in SDP Record.\r\n"));
				}
			} else {
				Display(
						("Unable to Open Server on: 1, Error = %d.\r\n", ret_val));

				ret_val = UNABLE_TO_REGISTER_SERVER;
			}

		} else {
			/* A Server is already open, this program only supports one    */
			/* Server or Client at a time.                                 */
			Display(("Server already open.\r\n"));

			ret_val = FUNCTION_ERROR;
		}
	} else {
		/* No valid Bluetooth Stack ID exists.                            */
		ret_val = INVALID_STACK_ID_ERROR;
	}

	return (ret_val);
}

/* The following function is responsible for setting the name of the */
/* local Bluetooth Device to a specified name.  This function returns*/
/* zero on successful execution and a negative value on all errors.  */
static int SetLocalNameAFib(void) {
	int Result;
	int ret_val = 0;
	char* name = "AFibDet Monitor";

	/* First, check that valid Bluetooth Stack ID exists.                */
	if (BluetoothStackID) {
		/* Attempt to submit the command.                              */
		Result = GAP_Set_Local_Device_Name(BluetoothStackID, name);

		/* Check the return value of the submitted command for success.*/
		if (!Result) {
			/* Display a messsage indicating that the Device Name was   */
			/* successfully submitted.                                  */

			/* Flag success to the caller.                              */
			ret_val = 0;
		} else {
			/* Display a message indicating that an error occured while */
			/* attempting to set the local Device Name.                 */

			ret_val = FUNCTION_ERROR;
		}

	} else {
		/* No valid Bluetooth Stack ID exists.                            */
		ret_val = INVALID_STACK_ID_ERROR;
	}

	return (ret_val);
}

/* send the default pin code */
static int defaultPINCodeResponse(void) {
	int Result;
	int ret_val;
	PIN_Code_t PINCode;
	GAP_Authentication_Information_t GAP_Authentication_Information;

	if (BluetoothStackID) {
		if (!COMPARE_BD_ADDR(CurrentRemoteBD_ADDR, NullADDR)) {
			ASSIGN_PIN_CODE(PINCode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
			BTPS_MemCopy(&PINCode, DEFAULT_PIN_CODE, BTPS_StringLength(DEFAULT_PIN_CODE));

			/* Populate the response structure */
			GAP_Authentication_Information.GAP_Authentication_Type = atPINCode;
			GAP_Authentication_Information.Authentication_Data_Length = (Byte_t) (BTPS_StringLength(DEFAULT_PIN_CODE));
			GAP_Authentication_Information.Authentication_Data.PIN_Code = PINCode;

			/* Submit the Authentication Response */
			if(!GAP_Authentication_Response(BluetoothStackID, CurrentRemoteBD_ADDR, &GAP_Authentication_Information)){
				ret_val = 0;
			} else {
				ret_val = FUNCTION_ERROR;
			}
			/* flag that we are not authenticating anymore */
			ASSIGN_BD_ADDR(CurrentRemoteBD_ADDR, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
		} else {
			ret_val = FUNCTION_ERROR;
		}
	} else {
		ret_val = INVALID_STACK_ID_ERROR;
	}
	return (ret_val);
}

/*********************************************************************/
/*                         Event Callbacks                           */
/*********************************************************************/

/* The following function is for the GAP Event Receive Data Callback.*/
/* This function will be called whenever a Callback has been         */
/* registered for the specified GAP Action that is associated with   */
/* the Bluetooth Stack.  This function passes to the caller the GAP  */
/* Event Data of the specified Event and the GAP Event Callback      */
/* Parameter that was specified when this Callback was installed.    */
/* The caller is free to use the contents of the GAP Event Data ONLY */
/* in the context of this callback.  If the caller requires the Data */
/* for a longer period of time, then the callback function MUST copy */
/* the data into another Data Buffer.  This function is guaranteed   */
/* NOT to be invoked more than once simultaneously for the specified */
/* installed callback (i.e.  this function DOES NOT have be          */
/* reentrant).  It Needs to be noted however, that if the same       */
/* Callback is installed more than once, then the callbacks will be  */
/* called serially.  Because of this, the processing in this function*/
/* should be as efficient as possible.  It should also be noted that */
/* this function is called in the Thread Context of a Thread that the*/
/* User does NOT own.  Therefore, processing in this function should */
/* be as efficient as possible (this argument holds anyway because   */
/* other GAP Events will not be processed while this function call is*/
/* outstanding).                                                     */
/* * NOTE * This function MUST NOT Block and wait for events that    */
/*          can only be satisfied by Receiving other GAP Events.  A  */
/*          Deadlock WILL occur because NO GAP Event Callbacks will  */
/*          be issued while this function is currently outstanding.  */
static void BTPSAPI GAP_Event_Callback(unsigned int BluetoothStackID,
		GAP_Event_Data_t *GAP_Event_Data, unsigned long CallbackParameter) {
	int Result;
	int Index;
	BD_ADDR_t NULL_BD_ADDR;
	GAP_Inquiry_Event_Data_t *GAP_Inquiry_Event_Data;
	GAP_Authentication_Information_t GAP_Authentication_Information;

	if ((BluetoothStackID) && (GAP_Event_Data)) {
		switch (GAP_Event_Data->Event_Data_Type) {
		case etInquiry_Result:
			GAP_Inquiry_Event_Data =
					GAP_Event_Data->Event_Data.GAP_Inquiry_Event_Data;

			if (GAP_Inquiry_Event_Data) {
				if (GAP_Inquiry_Event_Data->GAP_Inquiry_Data) {
					for (Index = 0;(Index < GAP_Inquiry_Event_Data->Number_Devices)
									&& (Index < MAX_INQUIRY_RESULTS); Index++) {
						InquiryResultList[Index] = GAP_Inquiry_Event_Data->GAP_Inquiry_Data[Index].BD_ADDR;

					}
					NumberofValidResponses = GAP_Inquiry_Event_Data->Number_Devices;
				}
			}
			break;
		case etInquiry_Entry_Result:
			break;
		case etAuthentication:
			switch (GAP_Event_Data->Event_Data.GAP_Authentication_Event_Data->GAP_Authentication_Event_Type) {
			case atLinkKeyRequest:
				/* Setup the authentication information response structure */
				GAP_Authentication_Information.GAP_Authentication_Type = atLinkKey;
				GAP_Authentication_Information.Authentication_Data_Length = 0;

				/* See if we have stored a Link Key for the specified device */
				for (Index = 0; Index < (sizeof(LinkKeyInfo) / sizeof(LinkKeyInfo_t)); Index++) {
					if (COMPARE_BD_ADDR(LinkKeyInfo[Index].BD_ADDR,
							GAP_Event_Data->Event_Data.GAP_Authentication_Event_Data->Remote_Device)) {
						/* Link Key information stored, respond with the stored Link Key */
						GAP_Authentication_Information.Authentication_Data_Length = sizeof(Link_Key_t);
						GAP_Authentication_Information.Authentication_Data.Link_Key = LinkKeyInfo[Index].LinkKey;
						break;
					}
				}
				/* Submit the authentication response */
				Result = GAP_Authentication_Response(BluetoothStackID,
						GAP_Event_Data->Event_Data.GAP_Authentication_Event_Data->Remote_Device,
						&GAP_Authentication_Information);
				break;
			case atPINCodeRequest:
				/* they want a pin code, give them the default pin code */
				/* TODO: have this not prompt for a pin code ?? */
				CurrentRemoteBD_ADDR = GAP_Event_Data->Event_Data.GAP_Authentication_Event_Data->Remote_Device;
				defaultPINCodeResponse();
				break;
			case atAuthenticationStatus:
				break;
			case atLinkKeyCreation:
				/* Store the link Key in either a free location OR over the old key location */
				ASSIGN_BD_ADDR(NULL_BD_ADDR, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
				for (Index = 0, Result = -1; Index < (sizeof(LinkKeyInfo) / sizeof(LinkKeyInfo_t)); Index++) {
					if (COMPARE_BD_ADDR(LinkKeyInfo[Index].BD_ADDR, GAP_Event_Data->Event_Data.GAP_Authentication_Event_Data->Remote_Device))
						break;
					else {
						if ((Result == (-1)) && (COMPARE_BD_ADDR(LinkKeyInfo[Index].BD_ADDR, NULL_BD_ADDR)))
							Result = Index;
					}
				}

				/* If we didn't find a match, see if we found an empty location */
				if (Index == (sizeof(LinkKeyInfo) / sizeof(LinkKeyInfo_t)))
					Index = Result;

				if (Index != (-1)) {
					LinkKeyInfo[Index].BD_ADDR = GAP_Event_Data->Event_Data.GAP_Authentication_Event_Data->Remote_Device;
					LinkKeyInfo[Index].LinkKey = GAP_Event_Data->Event_Data.GAP_Authentication_Event_Data->Authentication_Event_Data.Link_Key_Info.Link_Key;
				}
				break;
			case atIOCapabilityRequest:
				GAP_Authentication_Information.GAP_Authentication_Type = atIOCapabilities;
				GAP_Authentication_Information.Authentication_Data_Length = sizeof(GAP_IO_Capabilities_t);
				GAP_Authentication_Information.Authentication_Data.IO_Capabilities.IO_Capability = icNoInputNoOutput;
				GAP_Authentication_Information.Authentication_Data.IO_Capabilities.MITM_Protection_Required = TRUE;
				GAP_Authentication_Information.Authentication_Data.IO_Capabilities.OOB_Data_Present = FALSE;

				/* Submit the Authentication Response */
				Result = GAP_Authentication_Response(BluetoothStackID, GAP_Event_Data->Event_Data.GAP_Authentication_Event_Data->Remote_Device, &GAP_Authentication_Information);
				break;
			case atIOCapabilityResponse:
				break;
			case atUserConfirmationRequest:
				CurrentRemoteBD_ADDR = GAP_Event_Data->Event_Data.GAP_Authentication_Event_Data->Remote_Device;
				/* Invoke JUST Works Process...                    */
				GAP_Authentication_Information.GAP_Authentication_Type = atUserConfirmation;
				GAP_Authentication_Information.Authentication_Data_Length = (Byte_t) sizeof(Byte_t);
				GAP_Authentication_Information.Authentication_Data.Confirmation =
				TRUE;

				/* Submit the Authentication Response.             */
				Display(
						("\r\nAuto Accepting: %l\r\n", GAP_Event_Data->Event_Data.GAP_Authentication_Event_Data->Authentication_Event_Data.Numeric_Value));

				Result =
						GAP_Authentication_Response(BluetoothStackID,
								GAP_Event_Data->Event_Data.GAP_Authentication_Event_Data->Remote_Device,
								&GAP_Authentication_Information);

				/* Flag that there is no longer a current          */
				/* Authentication procedure in progress.           */
				ASSIGN_BD_ADDR(CurrentRemoteBD_ADDR, 0x00, 0x00, 0x00, 0x00,
						0x00, 0x00)
				;
				break;
			case atPasskeyRequest:

				/* Note the current Remote BD_ADDR that is requesting */
				/* the Passkey.                                       */
				CurrentRemoteBD_ADDR =
						GAP_Event_Data->Event_Data.GAP_Authentication_Event_Data->Remote_Device;
				break;
			case atRemoteOutOfBandDataRequest:
				/* This application does not support OOB data so      */
				/* respond with a data length of Zero to force a      */
				/* negative reply.                                    */
				GAP_Authentication_Information.GAP_Authentication_Type =
						atOutOfBandData;
				GAP_Authentication_Information.Authentication_Data_Length = 0;

				Result =
						GAP_Authentication_Response(BluetoothStackID,
								GAP_Event_Data->Event_Data.GAP_Authentication_Event_Data->Remote_Device,
								&GAP_Authentication_Information);
				break;
			case atPasskeyNotification:
				break;
			case atKeypressNotification:
				break;
			default:
				break;
			}
			break;
		case etRemote_Name_Result:
			/* Bluetooth Stack has responded to a previously issued     */
			/* Remote Name Request that was issued.                     */
			break;
		case etEncryption_Change_Result:
			break;
		default:
			/* An unknown/unexpected GAP event was received.            */
			break;
		}
	} else {
		/* There was an error with one or more of the input parameters.   */
	}
}

/* The following function is for an SPP Event Callback.  This        */
/* function will be called whenever a SPP Event occurs that is       */
/* associated with the Bluetooth Stack.  This function passes to the */
/* caller the SPP Event Data that occurred and the SPP Event Callback*/
/* Parameter that was specified when this Callback was installed.    */
/* The caller is free to use the contents of the SPP SPP Event Data  */
/* ONLY in the context of this callback.  If the caller requires the */
/* Data for a longer period of time, then the callback function MUST */
/* copy the data into another Data Buffer.  This function is         */
/* guaranteed NOT to be invoked more than once simultaneously for the*/
/* specified installed callback (i.e.  this function DOES NOT have be*/
/* reentrant).  It Needs to be noted however, that if the same       */
/* Callback is installed more than once, then the callbacks will be  */
/* called serially.  Because of this, the processing in this function*/
/* should be as efficient as possible.  It should also be noted that */
/* this function is called in the Thread Context of a Thread that the*/
/* User does NOT own.  Therefore, processing in this function should */
/* be as efficient as possible (this argument holds anyway because   */
/* another SPP Event will not be processed while this function call  */
/* is outstanding).                                                  */
/* * NOTE * This function MUST NOT Block and wait for Events that    */
/*          can only be satisfied by Receiving SPP Event Packets.  A */
/*          Deadlock WILL occur because NO SPP Event Callbacks will  */
/*          be issued while this function is currently outstanding.  */
static void BTPSAPI SPP_Event_Callback(unsigned int BluetoothStackID,
		SPP_Event_Data_t *SPP_Event_Data, unsigned long CallbackParameter) {
	int ret_val = 0;
	int sent;

	if ((SPP_Event_Data) && (BluetoothStackID)) {
		/* The parameters appear to be semi-valid, now check to see what  */
		/* type the incoming event is.                                    */
		switch (SPP_Event_Data->Event_Data_Type) {
		case etPort_Open_Indication:
			/* A remote port is requesting a connection.                */
			/* Save the Serial Port ID for later use.                   */
			SerialPortID =
					SPP_Event_Data->Event_Data.SPP_Open_Port_Indication_Data->SerialPortID;

			/* Query the connection handle.                             */
			ret_val =
					GAP_Query_Connection_Handle(BluetoothStackID,
							SPP_Event_Data->Event_Data.SPP_Open_Port_Indication_Data->BD_ADDR,
							&Connection_Handle);
			if (ret_val) {
				/* Failed to Query the Connection Handle.                */
				ret_val = 0;
				Connection_Handle = 0;
			} else {
				// start the adc
				startADC();
				// add the print function
				if (!BTPS_AddFunctionToScheduler(printADC, NULL, 0)) {
					// TODO something
					break;
				}
			}
			break;
		case etPort_Open_Confirmation:
			/* A Client Port was opened.  The Status indicates the      */
			/* Status of the Open.                                      */

			/* Check the Status to make sure that an error did not      */
			/* occur.                                                   */
			if (SPP_Event_Data->Event_Data.SPP_Open_Port_Confirmation_Data->PortOpenStatus) {
				/* An error occurred while opening the Serial Port so    */
				/* invalidate the Serial Port ID.                        */
				SerialPortID = 0;
				Connection_Handle = 0;
			} else {
				/* Query the connection Handle.                          */
				ret_val = GAP_Query_Connection_Handle(BluetoothStackID,
						SelectedBD_ADDR, &Connection_Handle);
				if (ret_val) {
					/* Failed to Query the Connection Handle.             */
					ret_val = 0;
					Connection_Handle = 0;
				}
			}
			break;
		case etPort_Close_Port_Indication:
			/* The Remote Port was Disconnected.                        */
			/* TODO: stop sending data */
			BTPS_DeleteFunctionFromScheduler(printADC, NULL);
			stopADC();

			Connection_Handle = 0;
			break;
		case etPort_Status_Indication:
			/* Display Information about the new Port Status.           */
			break;
		case etPort_Data_Indication:
			/* TODO: we recieved data, se need to process */
			break;
		case etPort_Send_Port_Information_Indication:
			/* Simply Respond with the information that was sent to us. */
			ret_val =
					SPP_Respond_Port_Information(BluetoothStackID,
							SPP_Event_Data->Event_Data.SPP_Send_Port_Information_Indication_Data->SerialPortID,
							&SPP_Event_Data->Event_Data.SPP_Send_Port_Information_Indication_Data->SPPPortInformation);
			break;
		case etPort_Transmit_Buffer_Empty_Indication:
			// only get when we try to send more data than we could, send what's left in buffer
			sent = SPP_Data_Write(BluetoothStackID, ServerPortID, BufferLength,
					Buffer);
			// TODO move data not sent
			break;
		default:
			/* An unknown/unexpected SPP event was received.            */
			break;
		}

		/* Check the return value of any function that might have been    */
		/* executed in the callback.                                      */
		if (ret_val) {
			/* An error occurred, so output an error message.              */
		}
	} else {
		/* There was an error with one or more of the input parameters.   */
	}
}

/* The following function is responsible for processing HCI Mode     */
/* change events.                                                    */
static void BTPSAPI HCI_Event_Callback(unsigned int BluetoothStackID,
		HCI_Event_Data_t *HCI_Event_Data, unsigned long CallbackParameter) {
	char *Mode;

	/* Make sure that the input parameters that were passed to us are    */
	/* semi-valid.                                                       */
	if ((BluetoothStackID) && (HCI_Event_Data)) {
		/* Process the Event Data.                                        */
		switch (HCI_Event_Data->Event_Data_Type) {
		case etMode_Change_Event:
			if (HCI_Event_Data->Event_Data.HCI_Mode_Change_Event_Data) {
				switch (HCI_Event_Data->Event_Data.HCI_Mode_Change_Event_Data->Current_Mode) {
				case HCI_CURRENT_MODE_HOLD_MODE:
					Mode = "Hold";
					break;
				case HCI_CURRENT_MODE_SNIFF_MODE:
					Mode = "Sniff";
					break;
				case HCI_CURRENT_MODE_PARK_MODE:
					Mode = "Park";
					break;
				case HCI_CURRENT_MODE_ACTIVE_MODE:
				default:
					Mode = "Active";
					break;
				}
			}
			break;
		}
	}
}

/* DO NOT call this with more than 256 bytes to send */
int serialSendData(Word_t length, Byte_t* data) {
	if (BluetoothStackID && ServerPortID) {
		int sent = SPP_Data_Write(BluetoothStackID, ServerPortID, length, data);
		if (sent < length && sent >= 0) {
			//we have data to send later
			BufferLength = length - sent;
			memcpy(Buffer, &(data[sent]), BufferLength);
		}
		return sent;
	}
	return -1;
}

/* The following function is used to initialize the application      */
/* instance.  This function should open the stack and prepare to     */
/* execute commands based on user input.  The first parameter passed */
/* to this function is the HCI Driver Information that will be used  */
/* when opening the stack and the second parameter is used to pass   */
/* parameters to BTPS_Init.  This function returns the               */
/* BluetoothStackID returned from BSC_Initialize on success or a     */
/* negative error code (of the form APPLICATION_ERROR_XXX).          */
int serialInitialize(HCI_DriverInformation_t *HCI_DriverInformation,
		BTPS_Initialization_t *BTPS_Initialization) {
	int ret_val = APPLICATION_ERROR_UNABLE_TO_OPEN_STACK;

	/* Initiailize some defaults.                                        */
	SerialPortID = 0;
	NumberofValidResponses = 0;

	/* Next, makes sure that the Driver Information passed appears to be */
	/* semi-valid.                                                       */
	if ((HCI_DriverInformation) && (BTPS_Initialization)) {
		/* Try to Open the stack and check if it was successful.          */
		if (!OpenStack(HCI_DriverInformation, BTPS_Initialization)) {
			/* The stack was opened successfully.  Now set some defaults.  */

			/* First, attempt to set the Device to be Connectable.         */
			ret_val = SetConnect();

			/* Next, check to see if the Device was successfully made      */
			/* Connectable.                                                */
			if (!ret_val) {
				/* Now that the device is Connectable attempt to make it    */
				/* Discoverable.                                            */
				ret_val = SetDisc();

				/* Next, check to see if the Device was successfully made   */
				/* Discoverable.                                            */
				if (!ret_val) {
					/* Now that the device is discoverable attempt to make it*/
					/* pairable.                                             */
					ret_val = SetPairable();
					if (!ret_val) {
						/* Attempt to register a HCI Event Callback.          */
						ret_val = HCI_Register_Event_Callback(BluetoothStackID,
								HCI_Event_Callback, (unsigned long) NULL);
						if (ret_val > 0) {
							/* Assign a NULL BD_ADDR for comparison.           */
							ASSIGN_BD_ADDR(NullADDR, 0x00, 0x00, 0x00, 0x00,
									0x00, 0x00);

							/* Open up the spp server on rfcomm port 1 */
							ret_val = OpenServerOnOne();

							if (ret_val >= 0) {
								/* set the name */
								ret_val = SetLocalNameAFib();

								if (ret_val >= 0) {
									/* Return success to the caller.                   */
									ret_val = (int) BluetoothStackID;
								}
							}
						} else {
							/* HCI_Register_Event_Callback() */
						}
					} else {
						/* SetPairable */
					}
				} else {
					/* SetDisc */
				}
			} else {
				/* SetDisc */
			}

			/* In some error occurred then close the stack.                */
			if (ret_val < 0) {
				/* Close the Bluetooth Stack.                               */
				CloseStack();
			}
		} else {
			/* There was an error while attempting to open the Stack.      */
			Display(("Unable to open the stack.\r\n"));
		}
	} else {
		ret_val = APPLICATION_ERROR_INVALID_PARAMETERS;
	}

	return (ret_val);
}

